/*
 * The Pipelined Regular Expression Monitor Compiler System
 * Copyright (c) 2002 University of British Columbia
 * All Rights Reserved.
 *
 *
 * Management of the symbol tables for PREMSC 
 *
 * Marcio Oliveira -- 2002
 */

#ifndef SYMBOLS_H
#define SYMBOLS_H

#include <dd.h>

#define SIGNAL_HASH_SIZE      257
#define INTERNAL_HASH_SIZE    257 
#define DEFINE_HASH_SIZE      257
#define PRODUCTION_HASH_SIZE  1039

#define P_SIGNAL_INTERNAL 0
#define P_SIGNAL_INPUT    1
#define P_SIGNAL_OUTPUT   2
#define P_SIGNAL_INOUT    3

typedef struct treeNode *treeNodePtr;
typedef struct indexNode *indexNodePtr;
 
typedef int signalType;

typedef struct signal *signalPtr;
struct signal {
    char *name;
    DdNode **dd;
    signalType type;
    int initValue;
    int index[2];
    int isArray;
    int srcline;
    indexNodePtr indexN;
    signalPtr next;
};

typedef struct define *definePtr; 
struct define {
    char *name;
    DdNode *dd;
    treeNodePtr expr;
    definePtr next;
    int srcline;
};

typedef struct production *productionPtr;
struct production {
    char *name;
    treeNodePtr prod;
    int srcline;
};

void initSymbols();
void insertSignal(signalPtr);
void insertInternal(signalPtr); 
void insertDefine(definePtr);
void insertProduction(productionPtr);
int lookupSignal(char *, signalPtr *);
int lookupInternal(char *, signalPtr *);
int lookupDefine(char *, definePtr *);
int lookupProduction(char *, productionPtr *);
void checkIndex(signalPtr, int);
int getOffset(signalPtr);

#endif
